/**
 * DevExtreme (file_management/remote_provider.js)
 * Version: 21.1.0 (build 21032-0316)
 * Build date: Mon Feb 01 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";

function _typeof(obj) {
    if ("function" === typeof Symbol && "symbol" === typeof Symbol.iterator) {
        _typeof = function(obj) {
            return typeof obj
        }
    } else {
        _typeof = function(obj) {
            return obj && "function" === typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj
        }
    }
    return _typeof(obj)
}
exports.default = void 0;
var _renderer = _interopRequireDefault(require("../core/renderer"));
var _ajax = _interopRequireDefault(require("../core/utils/ajax"));
var _common = require("../core/utils/common");
var _guid = _interopRequireDefault(require("../core/guid"));
var _window = require("../core/utils/window");
var _iterator = require("../core/utils/iterator");
var _deferred = require("../core/utils/deferred");
var _events_engine = _interopRequireDefault(require("../events/core/events_engine"));
var _provider_base = _interopRequireDefault(require("./provider_base"));
var _data2 = require("../core/utils/data");
var _type = require("../core/utils/type");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        })
    } else {
        obj[key] = value
    }
    return obj
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}

function _inherits(subClass, superClass) {
    if ("function" !== typeof superClass && null !== superClass) {
        throw new TypeError("Super expression must either be null or a function")
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        _setPrototypeOf(subClass, superClass)
    }
}

function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function(o, p) {
        o.__proto__ = p;
        return o
    };
    return _setPrototypeOf(o, p)
}

function _createSuper(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct();
    return function() {
        var result, Super = _getPrototypeOf(Derived);
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget)
        } else {
            result = Super.apply(this, arguments)
        }
        return _possibleConstructorReturn(this, result)
    }
}

function _possibleConstructorReturn(self, call) {
    if (call && ("object" === _typeof(call) || "function" === typeof call)) {
        return call
    }
    return _assertThisInitialized(self)
}

function _assertThisInitialized(self) {
    if (void 0 === self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return self
}

function _isNativeReflectConstruct() {
    if ("undefined" === typeof Reflect || !Reflect.construct) {
        return false
    }
    if (Reflect.construct.sham) {
        return false
    }
    if ("function" === typeof Proxy) {
        return true
    }
    try {
        Date.prototype.toString.call(Reflect.construct(Date, [], (function() {})));
        return true
    } catch (e) {
        return false
    }
}

function _getPrototypeOf(o) {
    _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function(o) {
        return o.__proto__ || Object.getPrototypeOf(o)
    };
    return _getPrototypeOf(o)
}
var window = (0, _window.getWindow)();
var FILE_CHUNK_BLOB_NAME = "chunk";
var FILE_SYSTEM_COMMNAD = {
    GET_DIR_CONTENTS: "GetDirContents",
    CREATE_DIR: "CreateDir",
    RENAME: "Rename",
    MOVE: "Move",
    COPY: "Copy",
    REMOVE: "Remove",
    UPLOAD_CHUNK: "UploadChunk",
    ABORT_UPLOAD: "AbortUpload",
    DOWLOAD: "Download"
};
var REQUEST_METHOD = {
    GET: "GET",
    POST: "POST"
};
var RemoteFileSystemProvider = function(_FileSystemProviderBa) {
    _inherits(RemoteFileSystemProvider, _FileSystemProviderBa);
    var _super = _createSuper(RemoteFileSystemProvider);

    function RemoteFileSystemProvider(options) {
        var _this;
        _classCallCheck(this, RemoteFileSystemProvider);
        options = (0, _common.ensureDefined)(options, {});
        _this = _super.call(this, options);
        _this._endpointUrl = options.endpointUrl;
        _this._beforeAjaxSend = options.beforeAjaxSend;
        _this._beforeSubmit = options.beforeSubmit;
        _this._requestHeaders = options.requestHeaders;
        _this._hasSubDirsGetter = (0, _data2.compileGetter)(options.hasSubDirectoriesExpr || "hasSubDirectories");
        return _this
    }
    _createClass(RemoteFileSystemProvider, [{
        key: "getItems",
        value: function(parentDir) {
            var _this2 = this;
            var pathInfo = parentDir.getFullPathInfo();
            return this._executeRequest(FILE_SYSTEM_COMMNAD.GET_DIR_CONTENTS, {
                pathInfo: pathInfo
            }).then((function(result) {
                return _this2._convertDataObjectsToFileItems(result.result, pathInfo)
            }))
        }
    }, {
        key: "renameItem",
        value: function(item, name) {
            return this._executeRequest(FILE_SYSTEM_COMMNAD.RENAME, {
                pathInfo: item.getFullPathInfo(),
                isDirectory: item.isDirectory,
                name: name
            })
        }
    }, {
        key: "createDirectory",
        value: function(parentDir, name) {
            return this._executeRequest(FILE_SYSTEM_COMMNAD.CREATE_DIR, {
                pathInfo: parentDir.getFullPathInfo(),
                name: name
            }).done((function() {
                if (parentDir && !parentDir.isRoot()) {
                    parentDir.hasSubDirectories = true
                }
            }))
        }
    }, {
        key: "deleteItems",
        value: function(items) {
            var _this3 = this;
            return items.map((function(item) {
                return _this3._executeRequest(FILE_SYSTEM_COMMNAD.REMOVE, {
                    pathInfo: item.getFullPathInfo(),
                    isDirectory: item.isDirectory
                })
            }))
        }
    }, {
        key: "moveItems",
        value: function(items, destinationDirectory) {
            var _this4 = this;
            return items.map((function(item) {
                return _this4._executeRequest(FILE_SYSTEM_COMMNAD.MOVE, {
                    sourcePathInfo: item.getFullPathInfo(),
                    sourceIsDirectory: item.isDirectory,
                    destinationPathInfo: destinationDirectory.getFullPathInfo()
                })
            }))
        }
    }, {
        key: "copyItems",
        value: function(items, destinationFolder) {
            var _this5 = this;
            return items.map((function(item) {
                return _this5._executeRequest(FILE_SYSTEM_COMMNAD.COPY, {
                    sourcePathInfo: item.getFullPathInfo(),
                    sourceIsDirectory: item.isDirectory,
                    destinationPathInfo: destinationFolder.getFullPathInfo()
                })
            }))
        }
    }, {
        key: "uploadFileChunk",
        value: function(fileData, chunksInfo, destinationDirectory) {
            var _data;
            if (0 === chunksInfo.chunkIndex) {
                chunksInfo.customData.uploadId = new _guid.default
            }
            var args = {
                destinationPathInfo: destinationDirectory.getFullPathInfo(),
                chunkMetadata: JSON.stringify({
                    UploadId: chunksInfo.customData.uploadId,
                    FileName: fileData.name,
                    Index: chunksInfo.chunkIndex,
                    TotalCount: chunksInfo.chunkCount,
                    FileSize: fileData.size
                })
            };
            var ajaxSettings = {
                url: this._endpointUrl,
                headers: this._requestHeaders || {},
                method: REQUEST_METHOD.POST,
                dataType: "json",
                data: (_data = {}, _defineProperty(_data, FILE_CHUNK_BLOB_NAME, chunksInfo.chunkBlob), _defineProperty(_data, "arguments", JSON.stringify(args)), _defineProperty(_data, "command", FILE_SYSTEM_COMMNAD.UPLOAD_CHUNK), _data),
                upload: {
                    onprogress: _common.noop,
                    onloadstart: _common.noop,
                    onabort: _common.noop
                },
                xhrFields: {},
                cache: false
            };
            var deferred = new _deferred.Deferred;
            this._beforeSendInternal(ajaxSettings);
            _ajax.default.sendRequest(ajaxSettings).done((function(result) {
                !result.success && deferred.reject(result) || deferred.resolve()
            })).fail(deferred.reject);
            return deferred.promise()
        }
    }, {
        key: "abortFileUpload",
        value: function(fileData, chunksInfo, destinationDirectory) {
            return this._executeRequest(FILE_SYSTEM_COMMNAD.ABORT_UPLOAD, {
                uploadId: chunksInfo.customData.uploadId
            })
        }
    }, {
        key: "downloadItems",
        value: function(items) {
            var args = this._getDownloadArgs(items);
            var $form = (0, _renderer.default)("<form>").css({
                display: "none"
            }).attr({
                method: REQUEST_METHOD.POST,
                action: args.url
            });
            var formDataEntries = {
                command: args.command,
                arguments: args.arguments
            };
            this._beforeSubmitInternal(formDataEntries);
            this._appendFormDataInputsToForm(formDataEntries, $form);
            $form.appendTo("body");
            _events_engine.default.trigger($form, "submit");
            setTimeout((function() {
                return $form.remove()
            }))
        }
    }, {
        key: "getItemsContent",
        value: function(items) {
            var args = this._getDownloadArgs(items);
            var ajaxSettings = {
                url: args.url,
                headers: this._requestHeaders || {},
                method: REQUEST_METHOD.POST,
                responseType: "arraybuffer",
                data: {
                    command: args.command,
                    arguments: args.arguments
                },
                upload: {
                    onprogress: _common.noop,
                    onloadstart: _common.noop,
                    onabort: _common.noop
                },
                xhrFields: {},
                cache: false
            };
            this._beforeSendInternal(ajaxSettings);
            return _ajax.default.sendRequest(ajaxSettings)
        }
    }, {
        key: "_getDownloadArgs",
        value: function(items) {
            var pathInfoList = items.map((function(item) {
                return item.getFullPathInfo()
            }));
            var args = {
                pathInfoList: pathInfoList
            };
            var argsStr = JSON.stringify(args);
            return {
                url: this._endpointUrl,
                arguments: argsStr,
                command: FILE_SYSTEM_COMMNAD.DOWLOAD
            }
        }
    }, {
        key: "_getItemsIds",
        value: function(items) {
            return items.map((function(it) {
                return it.relativeName
            }))
        }
    }, {
        key: "_executeRequest",
        value: function(command, args) {
            var method = command === FILE_SYSTEM_COMMNAD.GET_DIR_CONTENTS ? REQUEST_METHOD.GET : REQUEST_METHOD.POST;
            var deferred = new _deferred.Deferred;
            var ajaxSettings = {
                url: this._getEndpointUrl(command, args),
                headers: this._requestHeaders || {},
                method: method,
                dataType: "json",
                data: {},
                xhrFields: {},
                cache: false
            };
            this._beforeSendInternal(ajaxSettings);
            _ajax.default.sendRequest(ajaxSettings).then((function(result) {
                !result.success && deferred.reject(result) || deferred.resolve(result)
            }), (function(e) {
                return deferred.reject(e)
            }));
            return deferred.promise()
        }
    }, {
        key: "_beforeSubmitInternal",
        value: function(formDataEntries) {
            if ((0, _type.isFunction)(this._beforeSubmit)) {
                this._beforeSubmit({
                    formData: formDataEntries
                })
            }
        }
    }, {
        key: "_beforeSendInternal",
        value: function(ajaxSettings) {
            if ((0, _type.isFunction)(this._beforeAjaxSend)) {
                var ajaxArguments = {
                    headers: ajaxSettings.headers,
                    formData: ajaxSettings.data,
                    xhrFields: ajaxSettings.xhrFields
                };
                this._beforeAjaxSend(ajaxArguments);
                ajaxSettings.headers = ajaxArguments.headers;
                ajaxSettings.data = ajaxArguments.formData;
                ajaxSettings.xhrFields = ajaxArguments.xhrFields
            }
            if ((0, _type.isEmptyObject)(ajaxSettings.data)) {
                delete ajaxSettings.data
            } else if (ajaxSettings.responseType || ajaxSettings.upload) {
                ajaxSettings.data = this._createFormData(ajaxSettings.data)
            }
        }
    }, {
        key: "_createFormData",
        value: function(formDataEntries) {
            var formData = new window.FormData;
            for (var entryName in formDataEntries) {
                if (Object.prototype.hasOwnProperty.call(formDataEntries, entryName) && (0, _type.isDefined)(formDataEntries[entryName])) {
                    formData.append(entryName, formDataEntries[entryName])
                }
            }
            return formData
        }
    }, {
        key: "_appendFormDataInputsToForm",
        value: function(formDataEntries, formElement) {
            for (var entryName in formDataEntries) {
                if (Object.prototype.hasOwnProperty.call(formDataEntries, entryName) && (0, _type.isDefined)(formDataEntries[entryName])) {
                    (0, _renderer.default)("<input>").attr({
                        type: "hidden",
                        name: entryName,
                        value: formDataEntries[entryName]
                    }).appendTo(formElement)
                }
            }
        }
    }, {
        key: "_getEndpointUrl",
        value: function(command, args) {
            var queryString = this._getQueryString({
                command: command,
                arguments: JSON.stringify(args)
            });
            var separator = this._endpointUrl && this._endpointUrl.indexOf("?") > 0 ? "&" : "?";
            return this._endpointUrl + separator + queryString
        }
    }, {
        key: "_getQueryString",
        value: function(params) {
            var pairs = [];
            var keys = Object.keys(params);
            for (var i = 0; i < keys.length; i++) {
                var key = keys[i];
                var value = params[key];
                if (void 0 === value) {
                    continue
                }
                if (null === value) {
                    value = ""
                }
                if (Array.isArray(value)) {
                    this._processQueryStringArrayParam(key, value, pairs)
                } else {
                    var pair = this._getQueryStringPair(key, value);
                    pairs.push(pair)
                }
            }
            return pairs.join("&")
        }
    }, {
        key: "_processQueryStringArrayParam",
        value: function(key, array, pairs) {
            var _this6 = this;
            (0, _iterator.each)(array, (function(_, item) {
                var pair = _this6._getQueryStringPair(key, item);
                pairs.push(pair)
            }))
        }
    }, {
        key: "_getQueryStringPair",
        value: function(key, value) {
            return encodeURIComponent(key) + "=" + encodeURIComponent(value)
        }
    }, {
        key: "_hasSubDirs",
        value: function(dataObj) {
            var hasSubDirs = this._hasSubDirsGetter(dataObj);
            return "boolean" === typeof hasSubDirs ? hasSubDirs : true
        }
    }, {
        key: "_getKeyExpr",
        value: function(options) {
            return options.keyExpr || "key"
        }
    }]);
    return RemoteFileSystemProvider
}(_provider_base.default);
var _default = RemoteFileSystemProvider;
exports.default = _default;
module.exports = exports.default;
module.exports.default = module.exports;
